--
--  MANAGE_AMPLITUDE.vhd
--
--  Manage the amplitude for each tone.
--
--        Copyright (C)2001 - 2002 SEILEBOST
--                   All rights reserved.
--
-- $Id: MANAGE_AMPLITUDE.vhd, v0.1 2002/30/04 00:00:00 SEILEBOST $
--

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity MANAGE_AMPLITUDE is
    Port ( CLK           : in   std_logic;
           CLK_DAC       : in   std_logic;
           CLK_TONE      : in   std_logic;
           RST           : in   std_logic;
           AMPLITUDE     : in   std_logic_vector(4 downto 0);
           AMPLITUDE_E   : in   std_logic_vector(3 downto 0);
           OUT_DAC       : out  std_logic );        
end MANAGE_AMPLITUDE;

architecture Behavioral of MANAGE_AMPLITUDE is

 signal AMPLITUDE_TMP : std_logic_vector(3 downto 0);
 signal IN_DATA       : std_logic_vector(7 downto 0);

 COMPONENT DAC is Port ( CLK_DAC : in std_logic;
                         RST     : in std_logic;
                         IN_DAC  : in std_logic_vector(7 downto 0);
                         OUT_DAC : out std_logic );
 END COMPONENT;


begin

U_DAC : DAC PORT MAP ( CLK_DAC => CLK_DAC,
                       RST     => RST,
                       IN_DAC  => IN_DATA,
                       OUT_DAC => OUT_DAC);

  PROCESS(CLK, RST,  AMPLITUDE_TMP, AMPLITUDE_E)
  BEGIN
       if (RST = '1') then  -- reset
          AMPLITUDE_TMP <= "0000";
          IN_DATA       <= "00000000";
       elsif rising_edge(CLK) then -- edge clock                
             if (AMPLITUDE(4) = '0') then
                if (CLK_TONE = '1') then
                   AMPLITUDE_TMP <= AMPLITUDE(3 downto 0);                
                else
                   AMPLITUDE_TMP <= "0000";
                end if;
             else
                AMPLITUDE_TMP <= AMPLITUDE_E;                
             end if;
       end if;

-- Each amplitude has an 1.5 db step from previous amplitude
       CASE AMPLITUDE_TMP IS            
        when "0000" => IN_DATA <= "00000000"; -- 0        
        when "0001" => IN_DATA <= "00010110"; -- 22
        when "0010" => IN_DATA <= "00011010"; -- 26
        when "0011" => IN_DATA <= "00011111"; -- 31
        when "0100" => IN_DATA <= "00100101"; -- 37
        when "0101" => IN_DATA <= "00101100"; -- 44
        when "0110" => IN_DATA <= "00110100"; -- 52
        when "0111" => IN_DATA <= "00111110"; -- 62
        when "1000" => IN_DATA <= "01001010"; -- 74
        when "1001" => IN_DATA <= "01011000"; -- 88
        when "1010" => IN_DATA <= "01101001"; -- 105
        when "1011" => IN_DATA <= "01110101"; -- 125
        when "1100" => IN_DATA <= "10011001"; -- 149
        when "1101" => IN_DATA <= "10110001"; -- 177
        when "1110" => IN_DATA <= "11010010"; -- 210
        when "1111" => IN_DATA <= "11111111"; -- 255
        when OTHERS => NULL;
       END CASE; 

  END PROCESS;

end Behavioral;
